using System;
using System.Collections.Specialized;
using TX_Text_Control_Words.Properties;
using System.Windows.Forms;
using System.IO;

namespace TX_Text_Control_Words
{
	public class FileHandler
    {
        //
		// File handling in TX Text Control Words
		//
		// The sample program works in 2 modes, Default and XML mode. XML mode is entered when 
		// an  XML file is loaded. An XML file can only be saved as XML, but can be exported to 
		// all other formats.
		//
		// These file types will be available in the Save and Export dialog boxes. Save types are
		// are read/write, whereas Export types are those which cannot be loaded back in. When
		// a file is exported, the original file name and type are preserved.
        //
        # region "  Class properties  "
        const TXTextControl.StreamType DefaultExportTypes = TXTextControl.StreamType.AdobePDF | TXTextControl.StreamType.CascadingStylesheet;
		const TXTextControl.StreamType DefaultSaveTypes   = TXTextControl.StreamType.All & ~DefaultExportTypes & ~TXTextControl.StreamType.XMLFormat;
		const TXTextControl.StreamType DefaultInsertTypes = TXTextControl.StreamType.All & ~DefaultExportTypes & ~TXTextControl.StreamType.XMLFormat;
		const TXTextControl.StreamType XMLModeSaveTypes   = TXTextControl.StreamType.XMLFormat;
		const TXTextControl.StreamType XMLModeExportTypes = TXTextControl.StreamType.All & ~XMLModeSaveTypes;
		const TXTextControl.StreamType XMLModeInsertTypes = TXTextControl.StreamType.PlainAnsiText | TXTextControl.StreamType.PlainText;


		private string m_DocumentFileName = "";
		private TXTextControl.StreamType m_DocumentStreamType;
		private TXTextControl.TextControl m_TextControl;

        private frmMain m_mainForm;
        private ToolStripMenuItem m_ClientMenu;
        private StringCollection FileList;
        private int maxFiles;
                
        private bool m_DocumentDirty;

		// CSS related options
		private string m_CSSFileName = "";
		private TXTextControl.CssSaveMode m_CSSSaveMode;

		// PDF related options
		private string m_PDFMasterPassword = "";
		private string m_PDFUserPassword = "";
		private TXTextControl.DocumentAccessPermissions m_PDFDocumentAccessPermissions = 
			TXTextControl.DocumentAccessPermissions.AllowAll;
        
        # endregion

        # region "  Properties  "

        public frmMain mainForm
        {
            set { mainForm = value; }
        }
        
        public TXTextControl.TextControl TextControl
		{
			set { m_TextControl = value; }
		}

		public string DocumentFileName
		{
			set { m_DocumentFileName = value; }
			get {return m_DocumentFileName; }
		}

        public ToolStripMenuItem RecentFilesMenuItem
        {
            set { m_ClientMenu = value;}
        }

		public bool InXMLMode
		{
			get { return (m_DocumentStreamType == TXTextControl.StreamType.XMLFormat); }
		}

		public bool DocumentDirty
		{
			get { return  m_DocumentDirty; }
			set { m_DocumentDirty = value; }
		}

		public string CSSFileName
		{
			get { return m_CSSFileName; }
			set { m_CSSFileName = value; }
		}

		public TXTextControl.CssSaveMode CSSSaveMode
		{
			get { return m_CSSSaveMode; }
			set { m_CSSSaveMode = value; }
		}

		public string PDFMasterPassword
		{
			set { m_PDFMasterPassword = value; }
			get {return m_PDFMasterPassword; }
		}
		public string PDFUserPassword
		{
			set { m_PDFUserPassword = value; }
			get {return m_PDFUserPassword; }
		}
		public TXTextControl.DocumentAccessPermissions PDFDocumentAccessPermissions
		{
			set { m_PDFDocumentAccessPermissions = value; }
			get {return m_PDFDocumentAccessPermissions; }
        }

        # endregion

        # region "  Constructor  "
        public FileHandler(frmMain mainForm) 
        {
            m_mainForm = mainForm;
            FileList = Properties.Settings.Default.MRUFiles;
            maxFiles = Properties.Settings.Default.MRUMaxFiles;
        }
        # endregion

        # region "  File methods and functions  "
        public bool FileOpen()
		{
            bool Succeeded = false;
            
            TXTextControl.LoadSettings LoadSettings = new TXTextControl.LoadSettings();
            try
            {
                if (m_DocumentFileName == "")
                    m_TextControl.Load(TXTextControl.StreamType.All, LoadSettings);
                else
                {
                    if (m_DocumentFileName.ToLower().EndsWith(".rtf"))
                        m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.RichTextFormat, LoadSettings);
                    else if (m_DocumentFileName.ToLower().EndsWith(".doc"))
                        m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.MSWord, LoadSettings);
                    else if (m_DocumentFileName.ToLower().EndsWith(".docx"))
                        m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.WordprocessingML, LoadSettings);
                    else if (m_DocumentFileName.ToLower().EndsWith(".txt"))
                        m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.PlainText, LoadSettings);
                    else if (m_DocumentFileName.ToLower().EndsWith(".htm") || m_DocumentFileName.EndsWith(".html"))
                        m_TextControl.Load(m_DocumentFileName, TXTextControl.StreamType.HTMLFormat, LoadSettings);
                }

                if (LoadSettings.LoadedFile != "")
                {
                    m_DocumentFileName = LoadSettings.LoadedFile;
                    m_DocumentStreamType = LoadSettings.LoadedStreamType;
                    m_DocumentDirty = false;
                    m_CSSFileName = LoadSettings.CssFileName;
                    m_CSSSaveMode = TXTextControl.CssSaveMode.None;
                    AddRecentFile(m_DocumentFileName);
                    Succeeded = true;
                }
                m_mainForm.Text = m_DocumentFileName;
            }
            catch (Exception x)
            {
                MessageBox.Show(x.Message, "Error when loading document", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Succeeded = false;
            }
            return Succeeded;
		}

		public void FileSave()
		{
			TXTextControl.SaveSettings SaveSettings = new TXTextControl.SaveSettings();

			SaveSettings.CssFileName = m_CSSFileName;
			SaveSettings.CssSaveMode = m_CSSSaveMode;

			if (DocumentFileName != null &&  DocumentFileName != "")
			{
				// save under same name and type
				m_TextControl.Save(DocumentFileName, m_DocumentStreamType, SaveSettings);
			}
			else
			{
				// save as..
				if(InXMLMode)
					m_TextControl.Save(XMLModeSaveTypes, SaveSettings);
				else
					m_TextControl.Save(DefaultSaveTypes, SaveSettings);
			}

			if(SaveSettings.SavedFile != "")
			{
				m_DocumentFileName = SaveSettings.SavedFile;
				m_DocumentStreamType = SaveSettings.SavedStreamType;
				m_DocumentDirty = false;
                m_mainForm.Text = m_DocumentFileName;
                AddRecentFile(m_DocumentFileName);
			}
            
		}

		public void FileSaveAs()
		{
			TXTextControl.SaveSettings SaveSettings = new TXTextControl.SaveSettings();

			SaveSettings.CssFileName = m_CSSFileName;
			SaveSettings.CssSaveMode = m_CSSSaveMode;

			if (InXMLMode)
				m_TextControl.Save(XMLModeSaveTypes, SaveSettings);
			else
				m_TextControl.Save(DefaultSaveTypes, SaveSettings);

			if (SaveSettings.SavedFile != "")
			{
				if (m_DocumentStreamType != SaveSettings.SavedStreamType)
				{
                    DialogResult SaveWarning = DialogResult.OK;
                    if (SaveSettings.SavedStreamType == TXTextControl.StreamType.HTMLFormat || SaveSettings.SavedStreamType == TXTextControl.StreamType.PlainText || SaveSettings.SavedStreamType == TXTextControl.StreamType.PlainAnsiText)
                    {
                        SaveWarning = MessageBox.Show("Some formatting features are possibly not supported by the used save format.\nThese features may be lost or degraded when you reload this document.\nClick OK to reload the document anyway.\nTo keep all formatting features, click Cancel, and then save the file in another file format.", "Warning", MessageBoxButtons.OKCancel, MessageBoxIcon.Warning);
                    }
                    if (SaveWarning == DialogResult.OK)
                    {
                        // reload file if saved in a different format
                        m_TextControl.ResetContents();
                        m_TextControl.Load(SaveSettings.SavedFile, SaveSettings.SavedStreamType);
                        m_DocumentFileName = SaveSettings.SavedFile;
                        m_DocumentStreamType = SaveSettings.SavedStreamType;
                        m_DocumentDirty = false;
                        m_mainForm.Text = m_DocumentFileName;
                        AddRecentFile(m_DocumentFileName);
                    }
				}
				
			}
		}

		public void Export()
		{
			TXTextControl.SaveSettings SaveSettings = new TXTextControl.SaveSettings();

			SaveSettings.UserPassword = m_PDFUserPassword;
			SaveSettings.MasterPassword = m_PDFMasterPassword;
			SaveSettings.DocumentAccessPermissions = m_PDFDocumentAccessPermissions;

			if (InXMLMode)
				m_TextControl.Save(XMLModeExportTypes,SaveSettings);
			else
				m_TextControl.Save(DefaultExportTypes, SaveSettings);
		}

		public void Insert()
		{
			TXTextControl.LoadSettings LoadSettings = new TXTextControl.LoadSettings();

            if (InXMLMode)
                m_TextControl.Append(XMLModeInsertTypes, LoadSettings, TXTextControl.AppendSettings.StartWithNewSection);
            else
                m_TextControl.Append(DefaultInsertTypes, LoadSettings, TXTextControl.AppendSettings.StartWithNewSection);
			if (LoadSettings != null && LoadSettings.LoadedFile != "")
				m_DocumentDirty = true;
        }

        #endregion

        # region "  Recent File Handling "

        public void UpdateMenu()
        {
            int i;
            // Clear current menu
            for (i = m_ClientMenu.DropDownItems.Count - 1; i >= 0; i--)
            {
                m_ClientMenu.DropDownItems.RemoveAt(i);
            }
            // Check all recent files if they still exist
            CheckFiles();
            // Setup Recent files menu
            for (i = 0; i < FileList.Count; i++)
            {
                ToolStripItem mnuItm = new ToolStripMenuItem();
                mnuItm.Text = GetFileName(FileList[i]);
                mnuItm.Tag = FileList[i];
                mnuItm.Click += new EventHandler(mnuItm_Click);
                
                m_ClientMenu.DropDownItems.Add(mnuItm);
            }
            // Insert Clear menu entry
            if (m_ClientMenu.DropDownItems.Count >= 1)
            {
                m_ClientMenu.Enabled = true;
                m_ClientMenu.DropDownItems.Add("-");
                ToolStripMenuItem clearListItm = new ToolStripMenuItem("Clear list");
                m_ClientMenu.DropDownItems.Add(clearListItm);
                clearListItm.Click += new EventHandler(clearListItm_Click);
            }
            else {m_ClientMenu.Enabled = false;}
        }

        void clearListItm_Click(object sender, EventArgs e)
        {
            FileList.Clear();
            UpdateMenu();
        }

        void mnuItm_Click(object sender, EventArgs e)
        {
            System.Windows.Forms.DialogResult Result;

            if (m_DocumentDirty)
            {
                Result = MessageBox.Show("Save changes to " + DocumentFileName + "?", "Question", MessageBoxButtons.YesNoCancel);
                if (Result == DialogResult.Yes)
                {
                    FileSave();
                }
            }
            ToolStripMenuItem itm = (ToolStripMenuItem)sender;
            int pos = itm.GetCurrentParent().Items.IndexOf(itm);
            if (pos >= 0 && pos < FileList.Count)
            {
                m_DocumentFileName = itm.Tag.ToString();
                FileOpen();
            }
        }

        /// <summary>
        /// Retrieves the file title from the file path
        /// </summary>
        string GetFileName(string path)
        {
            int p1 = path.LastIndexOf(@"\");
            int p2 = path.LastIndexOf("/");
            p1 = Math.Max(p1, p2);
            int p3 = path.LastIndexOf(".");
            string rtn;
            if (p3 > p1) rtn = path.Substring(p1 + 1, p3 - p1 - 1);
            else rtn = path.Substring(p1 + 1);
            return rtn;
        }

        /// <summary>
		/// Adds a new file path to the top of the list
		/// </summary>
        void AddRecentFile (string filePath)
        {
            FileList.Insert(0, filePath);

            for (int last = FileList.Count - 1; last > 0; last--)
            {
                for (int frst = 0; frst < last; frst++)
                {
                    if (FileList[last] == FileList[frst])
                    {
                        FileList.RemoveAt(last);
                        break;
                    }
                }
            }
            TrimList();
            UpdateMenu();
        }

        void TrimList()
        {
            for (int bynd = FileList.Count - 1; bynd > maxFiles - 1; bynd--)
            {
                FileList.RemoveAt(bynd);
            }
        }

        public void InitRecentFiles ()
        {
            FileList = Properties.Settings.Default.MRUFiles;
            maxFiles = Properties.Settings.Default.MRUMaxFiles;
            UpdateMenu();
        }

        void CheckFiles()
        {
            for (int i = 0; i < FileList.Count; i++)
            {
                if(!File.Exists(FileList[i]))
                {
                    FileList.Remove(FileList[i]);
                }
                
            }
        }
        # endregion
    }
}
